// e3d.h - header file for 3d3 model editor
//
// changed 10/6/1999:
// updated poly defn to handle include a single colour for the polygon,
// and colour values for poly verts (more compatible).
// also, poly type defn (refers to Allegro poly types :)
#ifndef TRUE
#define TRUE	1
#define FALSE	0
#endif

#define E3D_VERSION		19
#define E3D_MODEL_VERSION	3
#define LIGHT_AMBIENT	1
#define LIGHT_DISTANT	2
#define LIGHT_OMNI		3

#define MAX_VERTS_PER_MODEL	2060
#define MAX_POLYS_PER_MODEL	2060
#define MAX_MODELS_PER_SCENE	2000
#define MAX_OBJECTS_PER_SCENE	2000
#define MAX_LIGHTS_PER_SCENE	10
#define MAX_CAMERAS_PER_SCENE	10

#define RAD2DEG			57.29578

// hicolour values
#define WHITE	65535
#define BLACK	0
#define YELLOW	65504
#define PURPLE	63519
#define CYAN	2047
#define RED		63488
#define GREEN	2016
#define BLUE	31
#define GREY	25356
#define DARK_GREY	16904

// for defining poly types for rendering compatibility later :)
// (stolen from allegro.h)
#define POLYTYPE_FLAT               0
#define POLYTYPE_GCOL               1
#define POLYTYPE_GRGB               2
#define POLYTYPE_ATEX               3
#define POLYTYPE_PTEX               4
#define POLYTYPE_ATEX_MASK          5
#define POLYTYPE_PTEX_MASK          6
#define POLYTYPE_ATEX_LIT           7
#define POLYTYPE_PTEX_LIT           8
#define POLYTYPE_ATEX_MASK_LIT      9
#define POLYTYPE_PTEX_MASK_LIT      10

// Set up structs
typedef struct {
	float x, y, z;
} Vector_f;

typedef struct {
	float x, y;
} Pt_f;

typedef struct {
	float r, g, b;
} rgb_f;

typedef struct {
	float x, y, z;
	float r, g, b;
    int group[5];
} Vertex_f;

typedef struct {
	short type, numverts;
	short p[6];							// vertex indices
	short n[6];							// normal indices
    rgb_f colour;						// polygon colour
    rgb_f vcol[6];						// vertex colours
    short shade;		// 0 = no shade, 1 = shade
    short gouraud;		// 0 = flat, 1 = gouraud
    short filling;		// 0 = solid colour, 1 = colour grad, 2 = texture
	short texmap;		// 1+ = texmap index
    Pt_f texp[6];
	float dist;
    int group[5];
} Pgon;


typedef struct {
	short numverts;
	Vertex_f vert[12];
    Pt_f texp[12];
} ClippedPgon;
 
typedef struct {
	Vertex_f pnt[12];
	short numverts;
    Pt_f texp[12];
	float dist;
} MappedPolygon;

typedef struct {
	Vertex_f pnt[12];
	Pt_f texp[12];
} ClippedMappedPolygon;

typedef struct {
	float x, y, z;
	float r, g, b;
	short type;
} ThreeDeeLight;

typedef struct {
	Vector_f posn;
	Vector_f dirn;
	Vector_f up;
	float viewangle;
} ThreeDeeCamera;

typedef struct {
    char name[20];
} GroupType;

typedef struct {
	char name[100];
	short numverts, numnorms, numpolys;
	Vertex_f vert[MAX_VERTS_PER_MODEL];
	Vector_f norm[MAX_VERTS_PER_MODEL];
	Pgon poly[MAX_POLYS_PER_MODEL];
	short shade;							// flag for shading
	float maxdim;							// max dimension for inst clip
    GroupType group[40];
    int numgroups;
} ThreeDeeModel;

typedef struct {
	short model, attach;
	short shade, active, visible, clipped;
	short preshade;
	rgb_f pscolour[30];						// inst preshade flag and preshaded poly colours
	Vector_f posn;
	Vector_f rotn;
	Vector_f scal;
	Vector_f dirn;							// dirn of motion
	float dist, maxdist;
	float speed, maxspeed, accel;			// speed of motion
	float turn, maxturn;					// manouvreability
	short energy;
	short behaviour, counter;				// behaviour stuff
    short nextbehaviour, nextcounter;		// behaviour stuff
	float tx, ty, tz;						// behaviour stuff
	short effect;							// special effect for instance
} ThreeDeeInstance;

typedef struct {
	int numlights;
	int nummodels;
	int numinsts;
	int numtexs;
	char texname[50][15];
} ThreeDeeScene;

typedef struct {			// need "ShowZones" routine
	int x1;					// and "GetZone" routine
	int y1;
	int x2;
	int y2;
	int active;				// may be used later
} MouseZone;

typedef struct {			// button definition
	int x, y;				// screen x and y posn
	int w, h;				// width and height of button
	int state;				// TRUE for down, FALSE for up
	int active;				// TRUE or FALSE
	int spr_on;				// sprite no index in datafile for "on" state
	int spr_off;			// ditto for "off" condition
	char text[40];
} GUIButton;

typedef struct {			// edit window
	int view;				// current view type - top, front or side
	float zoom;				// zoom factor
	float pan_x, pan_y;	// offset of window centre
	int grid;				// grid on/off
} EditWindow;

typedef struct {
	int r, g, b;
} EditRGB;

// *** for TIM's
typedef struct {
	int r;
    int g;
    int b;
    unsigned char alpha;
} RGB_Entry;

typedef struct {
	char fname[80];					// TIM file name
    int x;							// texture origin x (mult. of 64)
    int y;							// tex origin y (0 or 256)
    int w;							// tex width
    int h;							// tex height
    int cx;							// clut origin x (mult. of 16)
    int cy;							// clut origin y
    int mode;						// clut mode (4/8/16bit)
    int ABR;						// semitransparency mode
} PSX_Texture;


