// E3D_Scene1.cpp: implementation of the E3D_Scene class.
//
//////////////////////////////////////////////////////////////////////
#include "stdio.h"
#include "string.h"
#include "E3D.h"
#include "E3D_Scene1.h"
#include "E3D_Camera.h"
#include "E3D_Light.h"
#include "E3D_Object.h"
#include "E3D_Model.h"


//////////////////////////////////////////////////////////////////////
// Construction/Destruction
//////////////////////////////////////////////////////////////////////

E3D_Scene::E3D_Scene()
{
	numModels = 0;
}

E3D_Scene::~E3D_Scene()
{

}

// clear scene
void E3D_Scene::Start()
{
	numObjects = 0;
	numLights = 0;
	numCameras = 0;
}

// add an model instance to this scene
// returns: model instance id
class E3D_Model * E3D_Scene::AddModel()
{
	pModel[numModels] = new E3D_Model;
	if(pModel[numModels] == NULL) printf("Could not create Model!\n");
	return pModel[numModels++];
}

// add an model to this scene
// returns: model instance id
class E3D_Model * E3D_Scene::AddModel(char *filename)
{
	pModel[numModels] = new E3D_Model;
	pModel[numModels]->Load(filename);

	return pModel[numModels++];
}

// find a model in this scene
// returns: model instance id
int E3D_Scene::FindModel(char *modelName)
{
	int i;

	for(i=0; i<numModels; i++) {
		if(strcmp(pModel[i]->name, modelName) == 0) {
			return i;
		}
	}

	return 0;
}

// add an object instance to this scene
// returns: object instance id
class E3D_Object * E3D_Scene::AddObject()
{
	pObject[numObjects] = new E3D_Object;
	if(pObject[numObjects] == NULL) printf("Could not create Object!\n");
	pObject[numObjects]->modelNumber = 0;
	return pObject[numObjects++];
}

// add an object instance to this scene
// returns: object instance id
class E3D_Object * E3D_Scene::AddObject(char *modelName)
{
	int n;
	pObject[numObjects] = new E3D_Object;
	n = FindModel(modelName);
	pObject[numObjects]->modelNumber = n;
	pObject[numObjects]->pModel = pModel[n];
	
	//pObject = new E3D_Object;
	return pObject[numObjects++];
}

// add a light to this scene
// returns: light instance id
class E3D_Light * E3D_Scene::AddLight()
{
	pLight[numLights] = new E3D_Light;
	return pLight[numLights++];
}

// add a camera to this scene
// returns: camera instance id
class E3D_Camera * E3D_Scene::AddCamera()
{
	pCamera[numCameras] = new E3D_Camera;
	return pCamera[numCameras++];
}

// set clipping near and far values for this scene
void E3D_Scene::SetClipping()
{

}

// render this scene to the screen
int E3D_Scene::Render()
{

	return 0;
}


/* 	Error codes (return values):
			0.	successful (OK)
			1. unrecognised extension
			2.	unrecognised command in 3do file
			3. end of file before end marker
			4. out of bounds error
			5. file open failure
			6. too many vertices
			7. too many polygons
            8. TMD file error
*/

// Load a .scn file
int E3D_Scene::Load(char *filename)
{
	int i, j;
	int validcmd, validparam, end;
	class E3D_Model * pM;
	class E3D_Object * pO;
	class E3D_Light * pL;
	class E3D_Camera * pC;

	float x, y, z, x1, y1, z1, x2, y2, z2, r, g, b, a;
	char scmd[80], stemp[80];
	char *ext;
	FILE *pfscene;
	FILE *plog;

	plog = fopen(".\\log.log", "w");

	fprintf(plog, "Opening scene file %s\n", filename);

	pfscene = fopen(filename, "r");
	// check if file open failed
	if(pfscene == NULL) {
		fprintf(plog, "Scene file open failed.\n");
        return(1);
	}

	// Parse scene file
	numLights = 0;
	numModels = 0;
	numObjects = 0;
	numTextures = 0;
	numCameras = 0;
	end = FALSE;
	while(!end) {
		// reset valid flags to trap errors
		validcmd = FALSE; validparam = FALSE;

		fscanf(pfscene, "%s", &scmd[0]);
		fprintf(plog, "Command is %s\n", scmd);
		if(scmd[0] == '!') {
			validcmd = TRUE;
			fprintf(plog, "Comment encountered. Scanning till eol.\n");
			// read till end of line (character 10)
			fscanf(pfscene, "%c", &stemp[0]);
			while(stemp[0] != 10) fscanf(pfscene, "%c", &stemp[0]);
		}

		// LIGHT
		if(strcmp(scmd, "light") == 0) {
			validcmd = TRUE;
			validparam = FALSE;
			i = numLights;
			pL = AddLight();
			// get type of light
			fscanf(pfscene, "%s", &stemp[0]);
			if(strcmp(stemp, "distant") == 0) {
				fprintf(plog, "distant light:  ");
				fscanf(pfscene, "%f %f %f %f %f %f", &x, &y, &z, &r, &g, &b);
				fprintf(plog, "xyz %3.3f %f3.3 %3.3f   rgb %3.3f %3.3f %3.3f\n", x, y, z, r, g, b);
				pL->type = LIGHT_DISTANT;
				pL->x = x;		// x, y, z is direction vector (unit)
				pL->y = y;
				pL->z = z;
				pL->r = r;
				pL->g = g;
				pL->b = b;
				validparam = TRUE;
			}
			else if(strcmp(stemp, "omni") == 0) {
				fprintf(plog, "omni light:  ");
				fscanf(pfscene, "%f %f %f %f %f %f", &x, &y, &z, &r, &g, &b);
				fprintf(plog, "xyz %3.3f %3.3f %3.3f   rgb %3.3f %3.3f %3.3f\n", x, y, z, r, g, b);
				pL->type = LIGHT_OMNI;
				pL->x = x;		// x, y, z is direction vector (unit)
				pL->y = y;
				pL->z = z;
				pL->r = r;
				pL->g = g;
				pL->b = b;
				validparam = TRUE;
			}
			else if(strcmp(stemp, "ambient") == 0) {
				fprintf(plog, "ambient light:  ");
				fscanf(pfscene, "%f %f %f", &r, &g, &b);
				fprintf(plog, "rgb %3.3f %3.3f %3.3f\n", r, g, b);
				pL->type = LIGHT_AMBIENT;
				pL->r = r;
				pL->g = g;
				pL->b = b;
				validparam = TRUE;
			}
			else {
				fprintf(plog, "Error - Light type not recognised! Exiting...\n");
				fclose(pfscene);
				return(2);		// error
			}
		}

		// CAMERA
		if(strcmp(scmd, "camera") == 0) {
			validcmd = TRUE;
			fprintf(plog, "Camera detected.\n");
			i = numCameras;
			pC = AddCamera();
			fscanf(pfscene, "%f %f %f %f %f %f %f %f %f %f",
				&x, &y, &z, &x1, &y1, &z1, &x2, &y2, &z2, &a);
			fprintf(plog, "xyz %3.3f %3.3f %3.3f   dirn %3.3f %3.3f %3.3f   orient %3.3f %3.3f %3.3f  va %3.3f\n",
				x, y, z, x1, y1, z1, x2, y2, z2, a);
			pC->x = x;
			pC->y = y;
			pC->z = z;
			pC->rx = x1;
			pC->ry = y1;
			pC->rz = z1;
			//pC->up.x = x2;
			//pC->up.y = y2;
			//pC->up.z = z2;
			pC->FOV = a;
		}

/*		// load in background bitmap
		if(strcmp(scmd, "background") == 0) {
			validcmd = TRUE;
			fprintf(stderr, "Background bitmap detected - ");
			fscanf(pfscene, "%s", &stemp[0]);
			fprintf(stderr, "%s\n",	stemp);
			ext = strupr(strrchr(stemp, '.'));
			fprintf(stderr, "bg file ext is %s\n", ext);
			// check if right filetype (bmp or pcx)
			if((strcmp(ext, ".BMP") == 0) || (strcmp(ext, ".PCX") == 0)) {
				scene->background = load_bitmap(stemp, paltemp);
				if(scene->background == NULL) {
					fprintf(stderr, "Error: background bitmap load failed!\n");
				}
			}
			else fprintf(stderr, "Error: background bitmap filetype incorrect!\n");
		}
*/
		// load in background colour
		if(strcmp(scmd, "backcolour") == 0) {
			validcmd = TRUE;
			fprintf(plog, "Background colour detected.\n");
			fscanf(pfscene, "%f %f %f", &r, &g, &b);
			fprintf(plog, "bg col %3.3f %3.3f %3.3f\n", r, g, b);
			SetBackColour(r, g, b);
		}

		// load in fog params
		if(strcmp(scmd, "fog") == 0) {
			validcmd = TRUE;
			fprintf(plog, "Fog command detected.\n");
			fscanf(pfscene, "%f %f %f %f", &r, &g, &b, &x);
			fprintf(plog, "fog %3.3f %3.3f %3.3f %4.3f\n", r, g, b, x);
			SetFogColour(r, g, b);
			SetFogDistance(x);
		}

		// load in renderdistance param
		if(strcmp(scmd, "renderdistance") == 0) {
			validcmd = TRUE;
			fprintf(plog, "Renderdistance command detected.\n");
			fscanf(pfscene, "%f", &x);
			fprintf(plog, "renderdistance %5.3f\n", x);
			if(x > 1.0)	renderDistance = x;
			else fprintf(plog, "Error: invalid renderdist!\n");
		}


		// load in texture bitmap
		if(strcmp(scmd, "texture") == 0) {
			validcmd = TRUE;
			fprintf(plog, "Texture bitmap detected - ");
			fscanf(pfscene, "%s", &stemp[0]);
			fprintf(plog, "%s\n",	stemp);
			ext = strupr(strrchr(stemp, '.'));
			fprintf(plog, "texture file ext is %s\n", ext);
			// check if right filetype (bmp or pcx)
			if((strcmp(ext, ".BMP") == 0) || (strcmp(ext, ".PCX") == 0)) {
				//scene->ptex[scene->numtex] = load_bitmap(stemp, paltemp);
//blit(scene->ptex[0], screen, 0, 0, 0, 0, 256, 256);		// copy upper left
				//if(scene->ptex[scene->numtex] == NULL) {
				//	fprintf(plog, "Error: texture bitmap load failed!\n");
				//}
			}
			else fprintf(plog, "Error: texture bitmap filetype incorrect!\n");
			// increment texture count anyway
			// (textures must be loaded into the right slot!)
			numTextures++;
		}
		// OBJECT
		if(strcmp(scmd, "object") == 0) {
			validcmd = TRUE;
			fprintf(plog, "Object detected.\n");
			fscanf(pfscene, "%s %f %f %f %f %f %f %f %f %f %d",
				&stemp[0], &x, &y, &z, &x1, &y1, &z1, &x2, &y2, &z2, &j);
			fprintf(plog, "obj %s\n   posn %3.3f %3.3f %3.3f\n   rotn %3.3f %3.3f %3.3f\n   scale %3.3f %3.3f %3.3f\n   flags %d\n",
				stemp, x, y, z, x1, y1, z1, x2, y2, z2, j);
			i = numObjects;
			pO = AddObject();
			// set instance parameters
			//pO->flags = j;
			pO->x = x;
			pO->y = y;
			pO->z = z;
			pO->rx = x1;
			pO->ry = y1;
			pO->rz = z1;
			pO->sx = x2;
			pO->sy = y2;
			pO->sz = z2;
			pO->visible = 1;				// init visible
			//pO->attach = NULL;

			// Check if 3D model has already been loaded
			// if so, then set model number index of object
			pO->pModel = NULL;
			for(i=0; i < numModels; i++) {
				if(strcmp(pModel[i]->name, stemp) == 0) {
					fprintf(plog, "Model already loaded - model no. %d\n", i);
                    pO->pModel = pModel[i];		// set model for this inst
					pO->modelNumber = i;
					i = numModels;
				}
			}	// next i
			// if model not in loaded list, load it
			if(pO->pModel == NULL) {
				// Add and load this model
				pM = AddModel(stemp);
				if( pM == NULL ) {
					fprintf(plog, "Error - Object load failed! Exiting...\n");
					fclose(pfscene);
					return(3);
				}
				else {
					fprintf(plog, "Loaded model %s...", stemp);
					pO->pModel = pM;
					pO->modelNumber = numModels - 1;
				}
			}
		}

/*
		// load in landscape model
		if(strcmp(scmd, "landscape") == 0) {
			validcmd = TRUE;
			fprintf(stderr, "Landscape detected.\n");
			fscanf(pfscene, "%s %f %f %f %f %f %f %f %f %f",
				&stemp[0], &x, &y, &z, &x1, &y1, &z1, &x2, &y2, &z2);
			fprintf(stderr, "land %s   posn %3.3f %3.3f %3.3f   rotn %3.3f %3.3f %3.3f   scale %3.3f %3.3f %3.3f\n",
				stemp, x, y, z, x1, y1, z1, x2, y2, z2);
			scene->pland = (ThreeDeeInstance *)malloc(sizeof(ThreeDeeInstance));
			// set instance parameters
			scene->pland->posn.x = x;
			scene->pland->posn.y = y;
			scene->pland->posn.z = z;
			scene->pland->rotn.x = x1;
			scene->pland->rotn.y = y1;
			scene->pland->rotn.z = z1;
			scene->pland->scal.x = x2;
			scene->pland->scal.y = y2;
			scene->pland->scal.z = z2;

			// load landscape model and set object model index
			// allocate mem for model
			scene->plandmodel = (ThreeDeeLandscape *)malloc(sizeof(ThreeDeeLandscape));
			// reset model
			//if(Load3DO(stemp, scene->pmodel[scene->nummodels])) {
			i = E3D_LoadLandscape(stemp, scene->plandmodel);
			fprintf(stderr, "Object load returned %d\n", i);
			if(i == 0) {
				fprintf(stderr, "Object loaded succesfully.\n");
				//scene->pland->model = scene->plandmodel;
				// calculate poly normals and centroids for landscape
				E3D_CalculateCentroids(scene->plandmodel);
			}
			else {
				fprintf(stderr, "Error - Landscape load failed! Exiting...\n");
				fclose(pfscene);
				return(3);
			}
		}
*/
		if(strcmp(scmd, "end") == 0) {
			validcmd = TRUE;
			fprintf(plog, "End of scene file.\n");
			fprintf(plog, "Loaded %d models, %d instances, %d textures, %d lights and %d cameras.\n",
				 numModels, numObjects, numTextures, numLights, numCameras);
			end = TRUE;
		}

		// check if no valid command found (ie: parse error)
		if(!validcmd) {
			fprintf(plog, "Error - Scene Command not recognised! Exiting...\n");
			fclose(pfscene);
			return(4);		// error
		}
	}

	fclose(pfscene);

	fclose(plog);

	return(0);		// success!


}



void E3D_Scene::SetBackColour(float r, float g, float b)
{
	backColour.r = r;
	backColour.g = g;
	backColour.b = b;
}

void E3D_Scene::SetFogColour(float r, float g, float b)
{
	fogColour.r = r;
	fogColour.g = g;
	fogColour.b = b;
}

void E3D_Scene::SetFogDistance(float z)
{

}
