/***************************************************************************
Fractal Landscape generator
***************************************************************************/

/*
 Notes:
 1. Need to subdivide a 1024 * 1024 landscape (1 million verts) into 
    4096 16*16 objects (256 verts each)
 2. Fractalus is like 3D "Defender" :?:?
 3. Perhaps have 2 "LOD" for lanscape objects
 4. How do we do the edges?

*/

#include <stdlib.h>
#include <time.h>
#include "fracland.h"
#include "E3D.h"
#include "E3D_Utility.h"

unsigned char Map[Depth][Width];	/* Fractal map height array */
unsigned char MapColour[Depth][Width][3];

/* Size of each map cell: */
#define BG_CELLX	32
#define BG_CELLY	32

/* max width and height of screen display: */
#define BG_WIDTH	1536
#define BG_HEIGHT	720
	
/* number of cells:  */
#define BG_NX		(BG_WIDTH/BG_CELLX)
#define BG_NY		(BG_HEIGHT/BG_CELLY)	


/* screen size: */
#define SCR_W		256
#define SCR_H		240
#define SCR_Z		32

int Rnd = 35673;


/*****************************************************************************
InitFract
		Init fractal map, setting random heights at 32*32 offsets.
*****************************************************************************/
void InitFract(void)
{
int x;
int y;
int r=64;


	/* Seed the random-number generator with current time so that
     * the numbers will be different every time we run.
	 */
	srand( (unsigned)time( NULL ) );

	// set entire map to 0
	for(y=0; y<Depth; y++) {
		for(x=0; x<Width; x++) {
			Map[y][x] = 0;
		}
	}

	// create seed points
	for (y=0;y<Depth/32;y++)
	{
		for (x=0;x<Width/32;x++)
		{
			r = ModifyRnd(-1, 32);
			Map[y*32][x*32] += r;
			if (Map[y*32][x*32] > MAXHEIGHT) Map[y*32][x*32] = MAXHEIGHT;
		}
	}
}


/*****************************************************************************
CreateFract
		Create fractal map using seed points.
*****************************************************************************/
void CreateFract (void)
{
int x;
int y;
int f;
int xs;
int ys;
int xhs;
int yhs;
int Size;

	Size=32;
	while(Size!=0)
	{
		for (y=0;y<Depth;y+=Size)
		{
			for (x=0;x<Width;x+=Size)
			{
				xs=x+Size;
				ys=y+Size;
				xhs=x+(Size/2);
				yhs=y+(Size/2);

				// get average of surrounding points
				f=Map[y][x];
				f+=Map[ys][x];
				f+=Map[ys][xs];
				f+=Map[y][xs];
				f/=4;

				Map[yhs][xhs]=ModifyRnd(f, Size);
				Map[y][xhs]=ModifyRnd(f, Size);
				Map[yhs][x]=ModifyRnd(f, Size);
				if (ys>=Depth)
					Map[ys][xhs]=ModifyRnd(f, Size);
			}
			Map[yhs][xs]=ModifyRnd(f, Size);
		}
	Size/=2;
	}


	// create sea level
	for(x=0; x < Width; x++) {
		for(y=0; y < Depth; y++) {
			if(Map[x][y] < SEA_LEVEL) Map[x][y] = 0;
			else Map[x][y] = Map[x][y] - SEA_LEVEL;
		}
	}


}

void ShadeFract(void) {
	int i, j;
	E3D_Utility *pUtil;
	float nx, ny, nz;
	float ct;

	pUtil = new E3D_Utility;

	// for every fractal poly, calculate angle between
	// light and the poly normal
	for(i=0; i < Width-1; i++) {
		for(j=0; j < Depth-1; j++) {
			pUtil->poly_unit_normal_f(1, ((float)Map[i+1][j+1] / 5) - ((float)Map[i][j] / 5), 1,
				1, ((float)Map[i][j+1] / 5) - ((float)Map[i][j] / 5), 0,
				&nx, &ny, &nz);
			ct = pUtil->cos_theta_f(nx, ny, nz, 0.0f, 1.0f, 0.0f);
			MapColour[i][j][0] = (unsigned char)(ct * 127);
			MapColour[i][j][1] = (unsigned char)(ct * 64);
			MapColour[i][j][2] = (unsigned char)(ct * 32);
			
		}
	}


	free(pUtil);
}

/*****************************************************************************
SetPolysRGB
		Set each poly's RGB, depending on:
		a) Is it water?
		b) Angle to light source
		c) Distance from camera
*****************************************************************************/
void SetPolysRGB(int ix,int iy,int mx,int my)
{
int xx,yy;
int f2,f,r,g,b;

	xx = (mx+ix);//&(Width-1);
	yy = (my+iy);//&(Depth-1);

	f=Map[yy][xx];	
	if (f<SEA_LEVEL)
	{
		r=0;
		g=0;
		b=(f*4);							/* Water - blue */
	}
	else
	{
		r=f;								/* Rocks */
		g=f/4;
		b=f/2;
		f2=Map[yy][xx-1]-f;			/* Add lighting */
		r+=f2*8;
		g+=f2*8;
		b+=f2*8;
		f2=Map[yy+1][xx]-f;
		g+=f2*4;
	}

/*
	r-=((BG_NY-iy)*2);				// Add distance lighting 
	g-=((BG_NY-iy)*2);
	b-=((BG_NY-iy)*3);
*/
	if (r>255)							/* Limit RGB */
		r=255;
	else if (r<0)
		r=0;
	if (g>255)
		g=255;
	else if (g<0)
		g=0;
	if (b>255)
		b=255;
	else if (b<0)
		b=0;

	MapColour[iy][ix][0] = r;
	MapColour[iy][ix][1] = g;
	MapColour[iy][ix][2] = b;

//	FRTab[iy][ix]=r;					/* Store RGB */
//	FGTab[iy][ix]=g;
//	FBTab[iy][ix]=b;
}


/*****************************************************************************
ModifyRnd
		Requires:
					n		-ve returns rand 0-255.
							+ve returns n+/- 0-4.
*****************************************************************************/
int ModifyRnd(int f, int range)
{
int r;

	if(f<0) f = rand() % 255;				/* Return 0-255 */
	else
	{
		f += (rand() % range) - range/2;		/* Original number +/- 0-4 */
		if (f<0) f=0;				/* Limit value */
		else if (f>MAXHEIGHT) f=MAXHEIGHT;
	}
	return (f);	
}


